# Copyright (c) 2025 Kim Jarvis TPF Software Services S.A. kim.jarvis@tpfsystems.com 
# This software is licensed under the MIT License. See the LICENSE file for details.
#
class Manager:
    def __init__(self):
        self.sudo = False
        self.su = False
        self.manager = "apk"
        self.package = ""
